/* packet-ciscoci.c
 *
 * $Id: packet-ciscoci.c 12115 2004-09-27 22:55:15Z guy $
 *
 * Copyright (c) 2000 by Ed Warnicke <hagbard@physics.rutgers.edu>
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1999 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* Include files */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "plugins/plugin_api.h"

#include "moduleinfo.h"

#include <stdio.h>
#include <stdlib.h>
#include <gmodule.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include <epan/packet.h>
#include <epan/addr_resolv.h>
#include <epan/prefs.h>
#include <epan/strutil.h>

#include "plugins/plugin_api_defs.h"

/* Define version if we are not building ethereal statically */

#ifndef ENABLE_STATIC
G_MODULE_EXPORT const gchar version[] = VERSION;
#endif

/* Define udp_port for lawful intercept */

#define UDP_PORT_CISCOCI 54544
#define PACKET_OFFSET_CISCOCI 1

void proto_reg_handoff_ciscoci(void);

/* Define the ciscoci proto */

static int proto_ciscoci = -1;

/* Define the tree for ciscoci */

static int ett_ciscoci = -1;

/* 
 * Here are the global variables associated with the preferences
 * for ciscoci
 */

static guint global_udp_port_ciscoci = UDP_PORT_CISCOCI;
static guint udp_port_ciscoci = UDP_PORT_CISCOCI;

static guint global_packet_offset_ciscoci = PACKET_OFFSET_CISCOCI;
static guint packet_offset_ciscoci = PACKET_OFFSET_CISCOCI;


/* A static handle for the ip dissector */
static dissector_handle_t ip_handle;


static void dissect_ciscoci(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {

  proto_tree *ti,*ciscoci_tree;
  tvbuff_t * next_tvb;

  /* Set the protocol column */
  if(check_col(pinfo->cinfo,COL_PROTOCOL)){
    col_add_str(pinfo->cinfo,COL_PROTOCOL,"CISCOCI");
  }

  /* Set the info column */
  if(check_col(pinfo->cinfo,COL_INFO)){
  }

  /* 
   *If we have a non-null tree (ie we are building the proto_tree
   * instead of just filling out the columns ), then add a PLCI
   * tree node and put a CCCID header element under it.
   */
  if(tree) {
    ti = proto_tree_add_item(tree,proto_ciscoci,tvb,0,0,FALSE);
    ciscoci_tree = proto_item_add_subtree(ti,ett_ciscoci);
  }

  /*
   * Hand off to the IP dissector.
   */
  next_tvb = tvb_new_subset(tvb,packet_offset_ciscoci,-1,-1);
  call_dissector(ip_handle,next_tvb,pinfo,tree);
}




void proto_register_ciscoci(void) {
  static hf_register_info hf[] = {};

  static gint *ett[] = {
    &ett_ciscoci,
  };

  module_t *ciscoci_module;

  proto_ciscoci = proto_register_protocol("Cisco Cable Intercept UDP Encap",
				       "CISCOCI","ciscoci");
  proto_register_field_array(proto_ciscoci,hf,array_length(hf));
  proto_register_subtree_array(ett,array_length(ett));

  ciscoci_module = prefs_register_protocol(proto_ciscoci,
					proto_reg_handoff_ciscoci);
  prefs_register_uint_preference(ciscoci_module, "udp_port",
				 "CISCOCI UDP Port",
				 "The UDP port on which "
				 "Cisco Cable Intercept "
				 "packets will be sent",
				 10,&global_udp_port_ciscoci);

  prefs_register_uint_preference(ciscoci_module, "packet_offset",
                                 "CISCOCI Packet Offset",
                                 "The Offset of the encapped packet",
                                 10,&global_packet_offset_ciscoci);

}


/* The registration hand-off routing */

void proto_reg_handoff_ciscoci(void) {
  static int ciscoci_initialized = FALSE;
  static dissector_handle_t ciscoci_handle;

  ip_handle = find_dissector("frame");

  if(!ciscoci_initialized) {
    ciscoci_handle = create_dissector_handle(dissect_ciscoci,proto_ciscoci);
    ciscoci_initialized = TRUE;
  } else {
    dissector_delete("udp.port",udp_port_ciscoci,ciscoci_handle);
  }

  udp_port_ciscoci = global_udp_port_ciscoci;
  packet_offset_ciscoci = global_packet_offset_ciscoci;
  
  dissector_add("udp.port",global_udp_port_ciscoci,ciscoci_handle);

}


/* Start the functions we need for the plugin stuff */

#ifndef ENABLE_STATIC

G_MODULE_EXPORT void plugin_reg_handoff(void){
  proto_reg_handoff_ciscoci();
}

G_MODULE_EXPORT void plugin_init(plugin_address_table_t *pat
#ifndef PLUGINS_NEED_ADDRESS_TABLE
_U_
#endif
){
  /* initialise the table of pointers needed in Win32 DLLs */
  plugin_address_table_init(pat);
  /* register the new protocol, protocol fields, and subtrees */
  if (proto_ciscoci == -1) { /* execute protocol initialization only once */
    proto_register_ciscoci();
  }
}

#endif

/* End the functions we need for plugin stuff */

